import requests
import json
import base64
import hashlib

def robot(key, data):
    # 企业微信机器人的 webhook
    # 开发文档 https://work.weixin.qq.com/api/doc#90000/90136/91770
    webhook = f"https://qyapi.weixin.qq.com/cgi-bin/webhook/send?key={key}"
    headers = {'content-type': 'application/json'}  # 请求头
    r = requests.post(webhook, headers=headers, data=json.dumps(data))
    r.encoding = 'utf-8'
    # print(f'执行内容:{data}, 参数:{r.text}')
    # print(f'webhook 发送结果:{r.text}')
    return r.text


def bot_push(key, data):
    if type(key)!=str:
        print(f'webhook参数错误1:{key}_data')
        return
    if  len(key) == 0:
        print(f'webhook参数错误2:{key}_data')
        return

    try:
        res = robot(key, data)
        print(f'webhook 发出完毕: {res}')
        return res
    except Exception as e:
        print(e)


def bot_push_text(key, msg,user_list=[]):
    # 发送文本
    webhook_data = {
        "msgtype": "text",
        "text": {
            "content": msg ,
            "mentioned_list" : user_list
        }
    }

    # 企业微信机器人发送
    bot_push(key, webhook_data)
    return None

def bot_push_markdown(key, title , msg,user_list=[]):
    # 发送文本
	
	
    webhook_data = {
        "msgtype": "markdown",
        "markdown": {
            "content": f'{title}\n<font color=\"comment\">{msg}</font>' ,
            "mentioned_list" : user_list
        }
    }

    # 企业微信机器人发送
    bot_push(key, webhook_data)
    return None

def bot_push_pic(key, pic_path):
    
    pic_obj =None
    with open(pic_path,'rb') as pic:
        pic_obj = pic.read()
    pic_64  =  base64.b64encode(pic_obj)
    pic_str = str(pic_64,'utf-8')
    pic_md5 = hashlib.md5(pic_obj).hexdigest()

    # 发送图片
    webhook_data = {
        "msgtype": "image",
        "image": {
            "base64": pic_str,
            "md5": pic_md5
        }
    }
    # 企业微信机器人发送
    bot_push(key, webhook_data)

    return None




def wx_robot(robot_list,msg,is_debug=False,user_list=[]):
    if is_debug:
        print(f'Debug robot:\n{robot_list}\n{msg},\n{user_list}')
        return
    # 虽然最大支持5120，但是太长了也没啥意义，就1024凑个整吧
    if len(msg)>1024:
        msg = msg[-1024:]
    for key in robot_list:
        bot_push_text(key,msg,user_list)
    



import time
def time_stamp():
	'''
	超精准时间戳
	'''
	return int(round(time.time() * 1000000))


def time_stamp_s():
	'''
	秒级时间戳
	'''
	return int(round(time.time()))

def time_stamp_ms():
	'''
	超精准时间戳
	'''
	return int(round(time.time() * 1000))


def time_for_read(delay_stamp_ms=0):
	'''
	提供一种可读性好,且较为精准的时间戳
	精确到ms
	delay_stamp_ms为延迟后X毫秒后的时间戳
	'''
	x = int(time.time() * 1000) % 1000

	return time.strftime(f"%H-%M-%S-{x}", time.localtime(time.time()+delay_stamp_ms/1000))
def time_for_read_away(t1,t2):
	'''
	辅助time_for_read进行差值计算,返回是ms
	'''
	tp1 = t1.split('-')
	tp2 = t2.split('-')
	return (int(tp2[0])-int(tp1[0]))*3600000+(int(tp2[1])-int(tp1[1]))*60000+(int(tp2[2])-int(tp1[2]))*1000+(int(tp2[3])-int(tp1[3]))
	



def get_now_time():
	return time.strftime("%Y-%m-%d_%H-%M-%S", time.localtime())
def get_now_date():
	return time.strftime("%Y-%m-%d", time.localtime())

def get_next_date(days=1):
	'''
	之前的时间，默认一天后,也可以负数呀
	'''
	return time.strftime("%Y-%m-%d", time.localtime(time.time()-86400*days))

def get_next_hour(hour=1):
	'''
	之前的时间，默认一小时前,也可以负数呀，还可以小数呀
	'''
	return time.strftime("%Y-%m-%d_%H-%M-%S", time.localtime(time.time()-3600*hour))

def countdown(sleep_time):
	print(f'countdown:{sleep_time}/{sleep_time}')
	for i in range(1,sleep_time+1):
		time.sleep(1)
		print(f'countdown:{sleep_time-i}/{sleep_time}')



import yaml
import re
import traceback
from rogue_tools import path_tool

'''
    配合proto_tool使用，通过这个来获得【extension_name和io接口名称】
    通过这两个参数，就可以引入proto导出的xxxxx_pb2.py。
'''
class YamlGrop():
    '''
    所有yaml的聚合类\n
    '''
    def __init__(self,folder) -> None:
        self.yaml_dic = {}
        self.yaml_name_list=[]
        sub_list = path_tool.get_sub_files(folder,full_path=True,include='.yaml',exclude='EnumDef.yaml')
        for sub_file in sub_list:
            yaml = YamlObj(sub_file)
            self.yaml_dic[yaml.ext_id]=yaml
            self.yaml_dic[yaml.name]=yaml
            self.yaml_name_list.append(yaml.name)

    def get_all_yaml_name_list(self):
        return self.yaml_name_list

    def get_yaml(self,ext_id_or_name):
        return self.yaml_dic.get(ext_id_or_name,None)

    def get_yaml_name(self,ext_id_or_name):
        '''
        尽量不要使用，使用get_yaml_info替代
        '''
        yaml = self.get_yaml(ext_id_or_name)
        return yaml.name if yaml else None


    def get_yaml_request_name(self,ext_id_or_name,cmd_id):
        '''
        尽量不要使用，使用get_yaml_info替代
        '''
        return self.yaml_dic[ext_id_or_name].get_request_name(cmd_id)

    def get_yaml_reply_name(self,ext_id_or_name,cmd_id):
        '''
        尽量不要使用，使用get_yaml_info替代
        '''
        return self.yaml_dic[ext_id_or_name].get_reply_name(cmd_id)

    def get_yaml_info(self,ext_id_or_name,cmd_id):
        '''
        获得一个接口的信息
        '''
        yaml = self.get_yaml(ext_id_or_name)
        if not yaml:
            return None,None,None
        yaml_name = yaml.name
        if not yaml_name:
            return None,None,None
        request_name = self.get_yaml_request_name(ext_id_or_name,cmd_id)
        reply_name = self.get_yaml_reply_name(ext_id_or_name,cmd_id)
        return yaml_name,request_name,reply_name
class YamlObj():
    '''
    #####################################################################
    #                         一个自定义yaml对象                         #    
    #####################################################################  
    '''
    def __init__(self,file_path):
        self.file_path=file_path
        self.cmd_dic={}
        
        try:
            dic:dict            = yaml.safe_load(open(self.file_path, encoding='utf-8'))
            self.name           = dic.get('name',None)
            self.ext_id         = dic.get('extensionId',None)
            yaml_cmd_list       = dic.get('cmdList',None)

            for cmd in yaml_cmd_list:
                yaml_cmd = YamlCmd(cmd)
                self.cmd_dic[yaml_cmd.index]=yaml_cmd

            
        except Exception as err:
            traceback.print_exc()
    def get_all_cmd(self):
        return self.cmd_dic
        
    def get_extension_name(self):
        return self.name
        
    def get_request_name(self,index:int):
        return self.cmd_dic[index].request

    def get_reply_name(self,index:int):
        return self.cmd_dic[index].reply

class YamlCmd():
    def __init__(self,cmd_line) -> None:

        self.rs_dic={}
        pattern = re.compile(r'[(]([0-9]+)[)]', re.I)   # re.I 表示忽略大小写
        m = pattern.search(cmd_line)
        self.index     = int(m.group()[1:-1])
        self.name      = yaml_cmd_handle(cmd_line,'void')
        self.request   = yaml_cmd_handle(cmd_line,'@in')
        self.reply     = yaml_cmd_handle(cmd_line,'@out')


def yaml_cmd_handle(src_str,start_str):
    pattern = re.compile(r'('+start_str+')([ ]+)([a-z]+)', re.I)   # re.I 表示忽略大小写
    m = pattern.search(src_str)
    if m:
        pattern2 = re.compile(r'([a-z]+)', re.I)
        s=pattern2.search(m.group()[len(start_str):])
        return s.group()
    return None







import os
import sys
import time
import traceback

from PyQt5 import QtCore
from PyQt5.QtWidgets import *
from concurrent.futures import ThreadPoolExecutor




class mainUI():
    def __init__(self,w,h,title='PyQt5 - UI by [rogue_tools]',is_center=True):
        
        self.ui_width           = w
        self.ui_height          = h
        self.pool               = ThreadPoolExecutor(max_workers=3)
        self.app                = QApplication(sys.argv)
        self.windows            = QWidget()
        self.windows.resize(self.ui_width,self.ui_height)
        self.windows.setWindowTitle(title)
        # 居中
        if is_center:
            qr = self.windows.frameGeometry()
            cp = QDesktopWidget().availableGeometry().center()
            qr.moveCenter(cp)
            self.windows.move(qr.topLeft())

    def show(self):
        #show()方法在屏幕上显示出widget组件
        self.windows.show()
        #循环执行窗口触发事件，结束后不留垃圾的退出，不添加的话新建的widget组件就会一闪而过
        sys.exit(self.app.exec_())
    
    def add_label(self,line_index,title, start_pos = (0,0),obj_w=200,obj_h=20):
        label = QLabel(self.windows)
        label.setGeometry(QtCore.QRect(start_pos[0] , obj_h * line_index+start_pos[1] , obj_w , obj_h))
        label.setText(title)

    def add_btn(self,line_index,title, start_pos = (0,0),call_func=None,obj_w=100,obj_h=20):
        #设置按钮并给按钮命名
        btn = QPushButton(title,self.windows)
        btn.setGeometry(start_pos[0] , obj_h * line_index+start_pos[1] , obj_w , obj_h)
        if call_func:
            btn.clicked.connect(call_func)
        return btn
        
    def add_input_editor(self,line_index,title, start_pos = (0,0) ,edit_text='',edit_tips='',obj_w_1=70,obj_w_2=200,obj_h = 20):
        # 显示标签
        label = QLabel(self.windows)
        label.setGeometry(QtCore.QRect(start_pos[0] , obj_h * line_index+start_pos[1] , obj_w_1 , obj_h))
        label.setText(title)
        # 输入框
        edit = QLineEdit(self.windows)
        edit.setPlaceholderText(str(edit_tips))
        edit.setText(str(edit_text))
        edit.setGeometry(QtCore.QRect(obj_w_1+start_pos[0] , obj_h * line_index+start_pos[1] , obj_w_2 , obj_h))
        return edit
    
    def msgbox(self,msg_str,title=''):
        QMessageBox.about(self.windows, title,msg_str)

if __name__ == '__main__':
    main_ui = mainUI(400,400)
    main_ui.add_btn((10,10),1,'test1')
    main_ui.add_btn((10,10),2,'test2')
    main_ui.add_input_editor((160,10),1,'测试一下','test1')
    main_ui.add_input_editor((160,10),2,'测试一下','test2')
    main_ui.add_label((10,100),1,'test1')
    main_ui.add_label((10,100),2,'test2')
    main_ui.show()


import os
import time
import traceback
from concurrent.futures import ThreadPoolExecutor
from rogue_tools import time_tool,win_tool,path_tool


class Android():
    def __init__(self) -> None:
        '''
        一般情况下，只连一个android可以连接
        '''
        self.device = None
        self.adb_path = 'rogue_tools\\adb\\adb.exe'
        self.default_scrcpy_window = 'scrcpy_window'
        #self.default_port = 5037
        self.device_list = self.get_device_list(60)
        if len(self.device_list)==0:
            raise Exception('\nWarning: \nNone android device has connected.')

    def get_device_list(self,wait_time=15):
        '''
        #获取已连接的设备
        '''
        rs_list=[]
        start_time = time_tool.time_stamp_s()
        while time_tool.time_stamp_s() - start_time < wait_time:
            f=os.popen(f'{self.adb_path} devices')
            cmd_rs=f.read()
            #print(f'get_device_list:{cmd_rs}')
            device_tup=cmd_rs.split('\n')
            if len(device_tup)>1 and len(device_tup[1]) > 4 :
                break
            time.sleep(1)
            f=os.popen(f'{self.adb_path} kill-server')
        for i in range(1,len(device_tup)):
            temp = device_tup[i][0:-7]
            if temp:
                rs_list.append(temp)

        return rs_list

    def set_device(self,device):
        if device in self.device_list:
            self.device = device
        else:
            raise Exception(f'DeviceError : [{device}] has not connected.')
        

    def adb_execute(self,cmd):
        adb = f'{self.adb_path} -s {self.device}' if self.device else 'adb'
        adb = f'{adb} {cmd}'
        print(f'*adb_execute:{adb}')
        rs = os.popen(adb)
        #time.sleep(1)
        return rs

    def install_apk(self,local_path , package_name , time_out=180 , is_replace = True,is_option_g=True):
        '''
        使用这个方法安装apk
        '''
        if is_replace and self.is_install(package_name):
            self.uninstall_apk(package_name)

        start_time = time_tool.time_stamp_s()
        pool = ThreadPoolExecutor(max_workers=3)
        try:
            future1 = pool.submit(self.__install_apk, local_path , is_option_g)
            while time_tool.time_stamp_s() - start_time < time_out:
                print(f'install_apk cost:{time_tool.time_stamp_s() - start_time}s')
                time.sleep(10)
                if self.is_install(package_name):
                    return time_tool.time_stamp_s() - start_time
            return -1
        except BaseException:
            traceback.print_exc()
        finally:
            pool.shutdown()
    def __install_apk(self,local_path,is_option_g):
        '''
        安装apk,并返回安装时长
        '''
        time_install=time_tool.time_stamp_s()
        cmd = f'install -g' if is_option_g else 'install'
        cmd = f'{cmd} {local_path}'
        try:
            self.adb_execute(cmd)
            return time_tool.time_stamp_s()-time_install
        except BaseException :
            traceback.print_exc()
            return -1

        
    def uninstall_apk(self,package_name):
        print(f'uninstalling:{package_name}')
        while self.is_install(package_name):
            self.adb_execute(f'uninstall {package_name}')
            time.sleep(3)


    def is_install(self,package_name):
        f=self.adb_execute('shell pm list packages')
        cmd_rs=f.read()
        if package_name in cmd_rs:
            return True
        else:
            return False

    def start_app(self,package_name):
        self.adb_execute(f'shell monkey -p {package_name} -v -v -v 1')
        
    def stop_app(self,package_name):
        self.adb_execute(f'shell am force-stop {package_name}')

    def rm(self,path):
        if path in ('/','/sdcard','/storage','/system'):
            raise Exception(f'rm error,for path = {path}')
        self.adb_execute(f'shell rm -rf {path}')
        
    def tar_folder(self,src_folder,target_full_path:str,wait_file = True):
        '''
        将某个文件夹打包
        '''
        #adb shell tar -cvf /sdcard/cache.tar /sdcard/cache
        target_full_path = target_full_path if target_full_path.endswith('.tar') else target_full_path + '.tar'
        self.adb_execute(f'shell tar -cvf {target_full_path} {src_folder}')
        # 要等待，就返回等待的结果，不等待就直接True
        if wait_file:
            return self.wait_file_prepare(target_full_path)
        return True
    def pull_file(self,file_or_files,local_folder,wait_file = True):
        '''
        拉取安卓文件,可能会报权限错误,这个无解
        一般先tar打包,再拉
        '''
        #adb pull /sdcard/Android/data/com.baitian.spacex.sx.bt/files/CustomAstralLog.log E:\\test
        if type(file_or_files)==list:
            for file in file_or_files:
                self.pull_file(file,local_folder)
        else:
            self.adb_execute(f'pull {file_or_files} {local_folder}')
            # 要等待，就返回等待的结果，不等待就直接True
            if wait_file:
                name = os.path.basename(file_or_files)
                return path_tool.wait_file_prepare(path_tool.join_path(local_folder,name))
            return True
    
    def get_file_size(self,file):
        #adb shell stat -c "%s"
        if file.endswith('/'):
            print(f'Warning!Your path [ {file} ] is folder,not file.Then you can see 3488')
        try:
            rs = self.adb_execute(f'shell stat -c "%s" {file}').read()
        except BaseException:
            rs = 0
            traceback.print_exc()
        return int(rs)

    def wait_file_prepare(self,file,time_out = 600):
        total = 0
        start_time = time_tool.time_stamp_s()
        while True and time_tool.time_stamp_s() - start_time < time_out:
            time.sleep(2)
            
            temp = self.get_file_size(file)
            print(f'waiting file:{time_tool.time_stamp_s() - start_time}s , {int(temp/1024)}kb')
            if temp == total and temp > 0:
                return True
            else:
                total = temp
        return False
    